@@ -74,7 +74,7 @@ noconstructor = 'static PyObject *\n' + \
 typetmpl = 'PyExtensionClass Py%(class)s_Type = {\n' + \
 	   '    PyObject_HEAD_INIT(NULL)\n' + \
 	   '    0,				/* ob_size */\n' + \
-	   '    "%(class)s",			/* tp_name */\n' + \
+	   '    "%(classname)s",		/* tp_name */\n' + \
 	   '    sizeof(PyGObject),		/* tp_basicsize */\n' + \
 	   '    0,				/* tp_itemsize */\n' + \
 	   '    /* methods */\n' + \
@@ -102,7 +102,7 @@ typetmpl = 'PyExtensionClass Py%(class)s_Type = {\n' + \
 interfacetypetmpl = 'PyExtensionClass Py%(class)s_Type = {\n' + \
 	   '    PyObject_HEAD_INIT(NULL)\n' + \
 	   '    0,				/* ob_size */\n' + \
-	   '    "%(class)s",			/* tp_name */\n' + \
+	   '    "%(classname)s",		/* tp_name */\n' + \
 	   '    sizeof(PyPureMixinObject),	/* tp_basicsize */\n' + \
 	   '    0,				/* tp_itemsize */\n' + \
 	   '    /* methods */\n' + \
@@ -430,7 +430,7 @@ def write_class(parser, objobj, overrides, fp=sys.stdout):
     fp.write('};\n\n')
 
     # write the type template
-    dict = { 'class': objobj.c_name }
+    dict = { 'class': objobj.c_name, 'classname': objobj.name }
     dict['getattr'] = write_getattr(parser, objobj, castmacro, overrides, fp)
     dict['methods'] = 'METHOD_CHAIN(_Py' + dict['class'] + '_methods)'
     fp.write(typetmpl % dict)
@@ -467,7 +467,8 @@ def write_interface(parser, interface, overrides, fp=sys.stdout):
     fp.write('};\n\n')
 
     # write the type template
-    dict = { 'class': interface.c_name, 'getattr': '0' }
+    dict = { 'class': interface.c_name, 'classname': interface.name,
+             'getattr': '0' }
     dict['methods'] = 'METHOD_CHAIN(_Py' + dict['class'] + '_methods)'
     fp.write(interfacetypetmpl % dict)
 
@@ -678,6 +679,19 @@ def write_functions(parser, overrides, prefix, fp=sys.stdout):
     fp.write(string.join(functions, ''))
     fp.write('};\n\n')
 
+def write_enums(parser, prefix, fp=sys.stdout):
+    if not parser.enums:
+        return
+    fp.write('\n/* ----------- enums and flags ----------- */\n\n')
+    fp.write('void\n' + prefix + '_add_constants(PyObject *module, const gchar *strip_prefix)\n{\n')
+    for enum in parser.enums:
+        if enum.deftype == 'enum':
+            fp.write('    pyg_enum_add_constants(module, %s, strip_prefix);\n'
+                     % (enum.typecode,))
+        else:
+            fp.write('    pyg_flags_add_constants(module, %s, strip_prefix);\n'
+                     % (enum.typecode,))
+    fp.write('}\n\n')
 
 def write_source(parser, overrides, prefix, fp=sys.stdout):
     fp.write('/* -*- Mode: C; c-basic-offset: 4 -*- */\n\n')
@@ -704,18 +718,19 @@ def write_source(parser, overrides, prefix, fp=sys.stdout):
 
     write_functions(parser, overrides, prefix, fp)
 
+    write_enums(parser, prefix, fp)
+
     fp.write('/* intialise stuff extension classes */\n')
     fp.write('void\n' + prefix + '_register_classes(PyObject *d)\n{\n')
     fp.write('    ExtensionClassImported;\n')
     fp.write(overrides.get_init() + '\n')
 
     for boxed in parser.boxes:
-        fp.write('    pyg_register_boxed(d, "' + boxed.c_name +
+        fp.write('    pyg_register_boxed(d, "' + boxed.name +
                  '", ' + boxed.typecode + ', &Py' + boxed.c_name + '_Type);\n')
     for interface in parser.interfaces:
-        uclass = string.lower(string.replace(interface.typecode, '_TYPE_', '_', 1))
-        fp.write('    pyg_register_interface(d, "' + interface.c_name +
-                 '", '+ uclass + '_get_type, &Py' + interface.c_name +
+        fp.write('    pyg_register_interface(d, "' + interface.name +
+                 '", '+ interface.typecode + ', &Py' + interface.c_name +
                  '_Type);\n')
     objects = parser.objects[:]
     pos = 0
@@ -733,16 +748,15 @@ def write_source(parser, overrides, prefix, fp=sys.stdout):
         if obj.parent != None:
             bases.append(obj.parent)
         bases = bases + obj.implements
-        uclass = string.lower(string.replace(obj.typecode, '_TYPE_', '_', 1))
         if bases:
             fp.write('    pygobject_register_class(d, "' + obj.c_name +
-                     '", ' + uclass + '_get_type, &Py' + obj.c_name +
+                     '", ' + obj.typecode + ', &Py' + obj.c_name +
                      '_Type, Py_BuildValue("(' + 'O' * len(bases) + ')", ' +
                      string.join(map(lambda s: '&Py'+s+'_Type', bases), ', ') +
                      '));\n')
         else:
             fp.write('    pygobject_register_class(d, "' + obj.c_name +
-                     '", ' + uclass + '_get_type, &Py' + obj.c_name +
+                     '", ' + obj.typecode + ', &Py' + obj.c_name +
                      '_Type, NULL);\n')
     fp.write('}\n')
 
