@@ -7,6 +7,10 @@ try:
 except ImportError:
     pass
 
+# TRUE and FALSE constants ...
+FALSE = 0
+TRUE  = 1
+
 # hack so that ltihooks is used when importing ExtensionClass ...
 import ExtensionClass
 del ExtensionClass
@@ -15,9 +19,9 @@ import gobject
 del gobject
 
 # load the required modules:
-from GTK import *
-import GDK
+import keysyms
 from _gtk import *
+import gdk # this is created by the _gtk import
 
 # old names compatibility ...
 mainloop = main
@@ -25,9 +29,9 @@ def mainquit(*args):
     main_quit()
 mainiteration = main_iteration
 
-load_font = GdkFont
-load_fontset = fontset_load
-create_pixmap = GtkPixmap
-create_pixmap_from_xpm = pixmap_create_from_xpm
-create_pixmap_from_xpm_d = pixmap_create_from_xpm_d
+load_font = gdk.Font
+load_fontset = gdk.fontset_load
+create_pixmap = gdk.Pixmap
+create_pixmap_from_xpm = gdk.pixmap_create_from_xpm
+create_pixmap_from_xpm_d = gdk.pixmap_create_from_xpm_d
 
